#pragma once

#include <string>
#include <vector>

#include <boost/property_tree/ptree.hpp>

#include <fcf/log.hpp>

#include "global_configuration.hpp"

namespace application {

  struct server_configuration_t;
  struct endpoint_configuration_t;

  struct server_connection_t{
    std::string name;
    std::string address;
    int         port;
  };

  struct handler_configuration_t{
    int                                     innerControlPort;
    std::string                             user;
    std::string                             password;
    int                                     listenQueue;
    int                                     clientSocketTimeout;
    int                                     maxRequestTimeout;
    int                                     serverSocketTimeout;
    int                                     maxResponseTimeout;
    int                                     startTimeout;
    int                                     stopTimeout;
    int                                     keepAliveTimeout;

    std::string                             name;
    std::string                             host;
    std::string                             path;
    int                                     processes = 1;
    int                                     processesLongRequests = 0;
    std::string                             script;
    int                                     maxMemoryLongRequest;
    int                                     maxMemory;
    std::vector<int>                        dataPorts;
    std::vector< std::vector<std::string> > nodeOptions;
    std::vector<std::string>                scriptArgs;
    void set_values(boost::property_tree::ptree& a_ptree, endpoint_configuration_t& a_owner);
  };

  struct endpoint_configuration_t {
    int                                   innerControlPort;
    std::string                           user;
    std::string                           password;
    int                                   listenQueue;
    int                                   clientSocketTimeout;
    int                                   maxRequestTimeout;
    int                                   serverSocketTimeout;
    int                                   maxResponseTimeout;
    int                                   startTimeout;
    int                                   stopTimeout;
    int                                   transmissionBufferSize;
    int                                   keepAliveTimeout;
    std::string                           bind4 = "none";
    std::string                           bind6 = "none";
    int                                   port  = 80;
    std::vector<handler_configuration_t>  handlers;

    void set_values(boost::property_tree::ptree& a_ptree, server_configuration_t& a_owner);
  };

  struct server_configuration_t {
    std::string                           serverName;
    std::string                           mainServer;
    std::vector<server_connection_t>      servers;
    std::string                           logFile;
    int                                   logLevel = fcf::LOG_LEVEL_LOG;
    int                                   logLifeTime = 30;
    int                                   innerControlPort = 0;

    std::string                           user;
    std::string                           password;
    int                                   listenQueue = 128;

    int                                   clientSocketTimeout = 5;
    int                                   maxRequestTimeout = 300;
    int                                   serverSocketTimeout = 5;
    int                                   maxResponseTimeout = 3600;

    int                                   startTimeout = 30;
    int                                   stopTimeout = 30;
    int                                   transmissionBufferSize = 1000000;
    int                                   keepAliveTimeout = 5;
    std::vector<endpoint_configuration_t> endpoints;

    void set_values(boost::property_tree::ptree& a_ptree, const global_configuration_t& a_gc);
  };


}
