#include <memory>
#include <map>
#include <thread>
#include <chrono>

#include <fcf/net/socket.hpp>
#include <fcf/net/protocol/http.hpp>


#include "configuration.hpp"
#include "subprocesses.hpp"

namespace application {

  class endpoint_t {
    public:
      endpoint_t(const server_configuration_t& a_server_config, const global_configuration_t& a_global_config, size_t a_endpoint_index);
      ~endpoint_t();
      void run();
      void stop();
      void restart_handler(std::string a_handler);
      void restart_handlers();
      void get_processes_for_message(std::vector<process_identification_t>& a_dst,
                                     bool a_only_active_processes,
                                     bool a_active_flag,
                                     int a_destenition_pair,
                                     std::string a_handler,
                                     std::string a_ignore_handler,
                                     int a_ignore_process);
      void release_processes_for_message(const std::vector<process_identification_t>& a_procs, bool a_active);

    private:
      void _endpoint_handler(fcf::net::socket_t* a_socket);
      void _handler(fcf::net::socket_sptr_t a_socket);
      void _buffered_forwarding(const char*                             a_unsent_data_buffer,
                                size_t                                  a_unsent_data_buffer_size,
                                unsigned long long                      a_total_unset_data_size,
                                fcf::net::socket_sptr_t                 a_client_socket,
                                fcf::net::socket_sptr_t                 a_process_socket,
                                int                                     a_handler_index,
                                int&                                    a_process_index,
                                std::chrono::system_clock::time_point   a_start_response,
                                fcf::net::protocol::http_header_info_t& a_request_header_info,
                                int                                     a_max_response_timeout,
                                char*                                   a_big_data_buffer);

      std::string                                 _server_name;
      std::string                                 _main_server_name;
      const endpoint_configuration_t              _config;
      const global_configuration_t                _global_config;
      std::mutex                                  _launch_mutex;
      std::thread                                 _endpoint_thread4;
      std::thread                                 _endpoint_thread6;
      bool                                        _stop;
      fcf::net::socket_sptr_t                     _socket4;
      fcf::net::socket_sptr_t                     _socket6;
      std::map<std::string, subprocesses_sptr_t>  _subprocesses;
  };

  typedef std::shared_ptr<endpoint_t> endpoint_sptr_t;

}
