#pragma once
#include <memory>
#include <boost/asio.hpp>

namespace application {

  class event_server_t;
  struct process_identification_t;

namespace event_server {

  enum EVENT_SERVER_LOCK_TYPE{
    ESLT_INNER_LOCAL,
    ESLT_INNER_GLOBAL,
    ESLT_EXTERNAL,
  };

  class locker_i {
  public:
    virtual ~locker_i();
    virtual void                         activate_connection(const process_identification_t& a_connection_identification) = 0;
    virtual void                         deactivate_connection(const process_identification_t& a_connection_identification) = 0;
    virtual boost::asio::awaitable<void> lock(EVENT_SERVER_LOCK_TYPE a_lock_type,
                                              const process_identification_t& a_connection_identification,
                                              const char* a_handler,
                                              const char* a_event_name,
                                              const char* a_event_lock_id,
                                              const char* a_key,
                                              unsigned short a_counter) = 0;
    virtual boost::asio::awaitable<void> unlock(EVENT_SERVER_LOCK_TYPE a_lock_type,
                                              const char* a_handler,
                                              const char* a_event_name,
                                              const char* a_event_lock_id,
                                              const char* a_key) = 0;
  };

  locker_i* create_locker(boost::asio::io_context& a_iocontext, event_server_t& a_event_server, const char* a_main_server_name);

}
}
