#pragma once
#include <vector>
#include <condition_variable>
#include <mutex>
#include <boost/asio.hpp>

namespace fcf {

  class condition_t {
  public:

    struct handler_info_t{
      std::function<void()>   lock;
      std::function<void()>   callback;
    };

    condition_t(boost::asio::io_context& a_iocontext)
      : _iocontext(a_iocontext){
    }

    void notify_all(){
      _condition.notify_all();
      std::vector<handler_info_t> descriptors(std::move(_descriptors));

      for(auto& descriptor : descriptors){
        boost::asio::co_spawn(_iocontext, [this, descriptor = std::move(descriptor)]() mutable {
          return this->_async_notify(descriptor);
        }, boost::asio::detached);
      }
    }

    template <typename Lock>
    void wait(Lock& a_lock){
      _condition.wait(a_lock);
    }

    template <typename Lock, typename Duration>
    void wait_for(Lock& a_lock, Duration a_duration){
      _condition.wait_for(a_lock, a_duration);
    }

    template <typename Lock, typename Handler = boost::asio::use_awaitable_t<> >
    boost::asio::awaitable<void> co_wait(Lock& a_lock, Handler&& a_handler = {}){
      auto initiate = [this, &a_lock]<typename TPHandler>(TPHandler&& a_ph) mutable {
        this->_co_wait(
          a_lock,
          [a_ph = std::make_shared<TPHandler>(std::forward<TPHandler>(a_ph))]() {
            (*a_ph)();
          }
        );
      };
      return boost::asio::async_initiate<Handler, void()>(initiate, a_handler);
    }

  private:

    boost::asio::awaitable<void> _async_notify(handler_info_t& a_descriptor){
      a_descriptor.lock();
      a_descriptor.callback();
      co_return;
    }

    template <typename Lock, typename Functor>
    void _co_wait(Lock& a_lock, Functor a_callback){
      handler_info_t hi{[&a_lock](){ a_lock.lock(); }, a_callback };
      _descriptors.push_back(hi);
      a_lock.unlock();
    }

    boost::asio::io_context&      _iocontext;
    std::vector<handler_info_t>   _descriptors;
    std::condition_variable_any   _condition;

  };
}
