#include <cstring>
#include <stdexcept>
#include <algorithm>

#include "http.hpp"
#include "../../string.hpp"

namespace fcf {
namespace net {
namespace protocol {

  std::string get_http_param(const char* a_data, int a_end, const char* a_param){
    a_end += 2;
    int start = fcf::find_stri(a_data, 0, a_end, a_param);
    if (start == -1)
      return std::string();
    start += (int)std::strlen(a_param);
    int end = fcf::find_str(a_data, start, a_end, "\r\n");
    if (end == -1)
      return std::string();
    while(start < end) {
      if ((unsigned char)a_data[start] > (unsigned char)' ')
        break;
      ++start;
    }
    return std::string(&a_data[start], &a_data[end]);
  }


  bool parse_http_header_request(const char* a_data, int a_end, http_header_info_t* a_dst){
    a_dst->header_end = fcf::find_str(a_data, 0, a_end, "\r\n\r\n");
    if (a_dst->header_end == -1)
      return false;

    int pos = fcf::find_str(a_data, 0, a_end, " ");
    if (pos == -1)
      throw std::runtime_error("Invalid request http header");

    int epos = fcf::find_str(a_data, pos+1, a_end, " ");
    if (epos == -1)
      throw std::runtime_error("Invalid request http header");
    a_dst->path = std::string(&a_data[pos+1], &a_data[epos]);

    std::string content_length = get_http_param(a_data, a_dst->header_end, "content-length:");
    a_dst->content_length = content_length.empty() ? 0 : std::stoull(content_length);

    std::string cookie = get_http_param(a_data, a_dst->header_end, "cookie:");
    size_t sbeg = cookie.find("___fcf___context=");
    if (sbeg != std::string::npos) {
      sbeg += 17;
      size_t send = cookie.find(";", sbeg);
      a_dst->context = cookie.substr(sbeg, send != std::string::npos ? send - sbeg : cookie.length() - sbeg);
    }

    if (a_dst->context.empty())
      a_dst->context = get_http_param(a_data, a_dst->header_end, "fcf-context:");

    a_dst->host = get_http_param(a_data, a_dst->header_end, "host:");
    return true;
  }

  bool parse_http_header_response(const char* a_data, int a_end, http_header_info_t* a_dst){
    a_dst->header_end = fcf::find_str(a_data, 0, a_end, "\r\n\r\n");
    if (a_dst->header_end == -1)
      return false;

    a_dst->path = std::string();

    a_dst->host = std::string();

    std::string content_length = get_http_param(a_data, a_dst->header_end, "content-length:");
    a_dst->content_length = content_length.empty() ? 0 : std::stoull(content_length);

    a_dst->connection = get_http_param(a_data, a_dst->header_end, "connection:");
    std::transform(a_dst->connection.begin(), a_dst->connection.end(), a_dst->connection.begin(), [](unsigned char c){ return std::tolower(c); });

    return true;
  }

} // protocol namespace 
} // net namespace 
} // fcf namespace 
