#ifdef WIN32
#include "details/socket_win.hpp"
#else
#include "details/socket_unix.hpp"
#endif

#include <boost/interprocess/sync/named_mutex.hpp>
#include <boost/interprocess/shared_memory_object.hpp>
#include <boost/interprocess/mapped_region.hpp>

#include "socket.hpp"
#include "../log.hpp"

namespace fcf {
namespace net {
namespace {
  static const std::string MUTEX_ID  = "5bac8e397e9c6071b8a38f15e9d3";
  static const std::string MEMORY_ID = "a1ae46205fa76de63f5b0b916d47";
}


  socket_t::~socket_t(){
  }

  void socket_t::send_all(const char* a_data, unsigned int a_size){
    int sc = 0;
    while(sc < a_size)
      sc += send(a_data, a_size);
  }

  socket_t* create_socket(const char* a_address, int a_port){
    return new details::socket_t(a_address, a_port);
  }

  socket_t* create_socket_attach(socket_handle_t a_socket_handle, const char* a_address, int a_port){
    return new details::socket_t(a_socket_handle, a_address, a_port);
  }

  unsigned short int _get_free_tcp_port___get_next_port(const std::vector< std::pair<unsigned short int, unsigned short int> >& a_ports, unsigned short int a_port){
    static std::vector< std::pair<unsigned short int, unsigned short int> > default_ports = {{20001, 20500}};
    const std::vector< std::pair<unsigned short int, unsigned short int> >& ports = !a_ports.empty() ? a_ports : default_ports;
    if (!a_port)
      return ports[0].first;

    for(size_t i = 0; i < ports.size(); ++i) {
      if (a_port + 1 >= ports[i].first && a_port + 1 <= ports[i].second)
        return a_port + 1;
    }

    int next_range_index = -1;
    for(size_t i = 0; i < ports.size(); ++i) {
      if (a_port + 1 < ports[i].first && (next_range_index == -1 || ports[i].first < ports[next_range_index].first))
        next_range_index = i;
    }
    if (next_range_index != -1)
      return ports[next_range_index].first;

    return ports[0].first;
  }

  void unlock_global_mutex(){
    try {
      boost::interprocess::named_mutex mutex(boost::interprocess::open_or_create, MUTEX_ID.c_str());
      mutex.unlock();
    } catch(...){
    }
  }

  int get_free_tcp_port(std::string a_needStr, const std::vector< std::pair<unsigned short int, unsigned short int> >& a_ports){
    std::vector< std::pair<unsigned short int, unsigned short int> > ports;
    int result = -1;

    try {
      boost::interprocess::named_mutex mutex(boost::interprocess::open_or_create, MUTEX_ID.c_str());
      std::unique_lock<boost::interprocess::named_mutex> lock(mutex);

      while(true) {
        unsigned short int port = 0;
        try {
          boost::interprocess::shared_memory_object smo(boost::interprocess::open_only, MEMORY_ID.c_str(), boost::interprocess::read_only);
          boost::interprocess::mapped_region mr(smo, boost::interprocess::read_only, 0, 2);
          port = *(unsigned short int*)mr.get_address();
        } catch(...) {
        }

        port = _get_free_tcp_port___get_next_port(a_ports, port);

        boost::interprocess::shared_memory_object smo(boost::interprocess::open_or_create, MEMORY_ID.c_str(), boost::interprocess::read_write);
        smo.truncate(2);
        boost::interprocess::mapped_region mr(smo, boost::interprocess::read_write, 0, 2);
        *(unsigned short int*)mr.get_address() = port;

        try {
          fcf::net::socket_sptr_t socket(fcf::net::create_socket("127.0.0.1", port));
          socket->set_reuse(true);
          socket->bind();
          socket->listen(1);
          result = port;
          break;
        } catch(...){
        }

      }
    } catch (std::exception& e) {
      fcf::err << "Get free port error: " << e.what();
    } catch (...) {
      fcf::err << "Get free port error: Unknow error";
    }

    if (result == -1)
      throw std::runtime_error(std::string() + "Failed to get free port for the \"" + a_needStr + "\"");

    fcf::log << "Found a free [" << result << "] port for the \"" << a_needStr << "\" ...";
    return result;
  }

} // net namespace
} // fcf namespace
