#pragma once
#include <memory>
#include <boost/asio.hpp>


namespace fcf {

  template <typename Duration, typename Handler = boost::asio::use_awaitable_t<> >
  boost::asio::awaitable<void> co_sleep(boost::asio::io_context& a_iocontext, Duration a_duration, Handler&& a_handler = {}){
    std::shared_ptr<boost::asio::deadline_timer> sptimer(new boost::asio::deadline_timer(a_iocontext, a_duration));
    auto init = [sptimer]<typename TPHandler>(TPHandler&& a_ph) mutable {
      sptimer->async_wait([sptimer, a_ph = std::make_shared<TPHandler>(std::forward<TPHandler>(a_ph))](const boost::system::error_code& e) mutable{
        (*a_ph)();
        sptimer.reset();
      });
    };
    return boost::asio::async_initiate<Handler, void()>(init, a_handler);
  };


}
