#ifdef WIN32
  #define _SILENCE_CXX17_CODECVT_HEADER_DEPRECATION_WARNING
  #define _SILENCE_ALL_CXX17_DEPRECATION_WARNINGS
  #pragma warning(disable:4996)
  #include <windows.h>
#else
  #include <unistd.h>
#endif

#ifdef __APPLE__
#include <sys/proc_info.h>
#include <libproc.h>
#endif

#include <codecvt>
#include <cstring>
#include <climits>
#include <filesystem>

#include "system.hpp"

namespace fcf {
  std::string get_system_error_str(int a_errno) {
    #ifdef WIN32
      std::string result;
      char* buffer;
      DWORD len = FormatMessageA(FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS,
                                 NULL, a_errno, MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), (LPSTR)&buffer, 0, NULL);
      if (len) {
        result.append(buffer, buffer + len);
        LocalFree(buffer);
      }
      return result;
    #else
      return strerror(a_errno);
    #endif
  }

  std::filesystem::path get_exe_directory() {
    std::filesystem::path result;
  #ifdef _WIN32
    wchar_t buffer[MAX_PATH] = { 0 };
    GetModuleFileNameW(0, buffer, MAX_PATH);
    result = std::filesystem::path(buffer);
  #elif __APPLE__
    char buffer[PROC_PIDPATHINFO_MAXSIZE] = { 0 };
    int res = proc_pidpath(getpid(), buffer, PROC_PIDPATHINFO_MAXSIZE);
    result = std::filesystem::path(buffer);
  #else
    char buffer[PATH_MAX] = { 0 };
    ssize_t count = readlink("/proc/self/exe", buffer, PATH_MAX);
    result = std::filesystem::path(std::string(buffer, (count > 0) ? count : 0));
  #endif
    return result.parent_path();
  }


}
