#define BOOST_BIND_GLOBAL_PLACEHOLDERS 1
#ifdef WIN32
#define _SILENCE_CXX17_CODECVT_HEADER_DEPRECATION_WARNING
#define _SILENCE_ALL_CXX17_DEPRECATION_WARNINGS
#endif
#ifndef WIN32
#include <csignal>
#include <unistd.h>
#endif

#include <time.h>
#include <stdlib.h>

#include <iostream>
#include <string>
#include <cstring>
#include <stdexcept>
#include <fstream>
#include <chrono>
#include <thread>
#include <streambuf>
#include <sstream>
#include <csignal>
#include <filesystem>

#include <boost/bind/bind.hpp>
#include <boost/property_tree/ptree.hpp>
#include <boost/property_tree/json_parser.hpp>

#include <fcf/system.hpp>
#include <fcf/format/json/json.hpp>
#include <fcf/log.hpp>
#include <fcf/net/net.hpp>
#include <fcf/net/socket.hpp>

#include "configuration.hpp"
#include "global_configuration.hpp"
#include "server.hpp"

application::server_sptr_t pserver_g;

void run(std::filesystem::path a_config_path) {
  srand(time(0));

  application::global_configuration_t gc;
  application::server_configuration_t sc;

  fcf::log << "--------------------------------------------";
  fcf::net::initialize();

  {
    std::ifstream file;
    try {
      file.exceptions(std::ifstream::failbit | std::ifstream::badbit);
      file.open(fcf::get_exe_directory() / "settings.json");
    } catch(std::exception& e) {
      throw std::runtime_error(std::string("Incorrect global config file (") + (fcf::get_exe_directory() / "settings.json").string() + "): Failed read file");
    }
    try {
      std::stringstream ss(fcf::format::json::adapt_json(std::string((std::istreambuf_iterator<char>(file)), std::istreambuf_iterator<char>())));
      boost::property_tree::ptree ptree;
      boost::property_tree::read_json(ss, ptree);
      gc.set_values(ptree);
    } catch(std::exception& e) {
      throw std::runtime_error(std::string("Incorrect global config file (") + (fcf::get_exe_directory() / "settings.json").string() + "): Failed format file");
    }
  }

  {
    bool custom_global_settings = false;
    std::ifstream file;
    try {
      file.exceptions(std::ifstream::failbit | std::ifstream::badbit);
      file.open(fcf::get_exe_directory() / "settings.custom.json");
      custom_global_settings = true;
    } catch(std::exception& e){
    }
    if (custom_global_settings) {
      try {
        std::stringstream ss(fcf::format::json::adapt_json(std::string((std::istreambuf_iterator<char>(file)), std::istreambuf_iterator<char>())));
        boost::property_tree::ptree ptree;
        boost::property_tree::read_json(ss, ptree);
        gc.set_values(ptree);
      } catch(std::exception& e){
        throw std::runtime_error(std::string("Incorrect global config file (") + (fcf::get_exe_directory() / "settings.custom.json").string() + "): Failed format file");
      }
    }
  }

  std::ifstream file;
  try {
    file.exceptions(std::ifstream::failbit | std::ifstream::badbit);
    file.open(a_config_path);
  } catch (std::exception& e) {
    throw std::runtime_error("Failed to read config file.");
  }
  try {
    std::stringstream ss(fcf::format::json::adapt_json(std::string((std::istreambuf_iterator<char>(file)), std::istreambuf_iterator<char>())));
    boost::property_tree::ptree conf_tree;
    boost::property_tree::read_json(ss, conf_tree);
    sc.set_values(conf_tree, gc);
  } catch(const std::exception& e) {
    throw std::runtime_error(std::string("Incorrect config file. ") + e.what());
  }

  fcf::log.get_logger().set_level(sc.logLevel);
  fcf::log.get_logger().set_file_prefix(sc.logFile);
  fcf::log.get_logger().set_life_time(sc.logLifeTime);

  fcf::log << "Start server";

  pserver_g.reset(new application::server_t(sc, gc));
  pserver_g->run();
}


#ifdef WIN32
  void int_signal_handler(int signum) {
    if (pserver_g.get())
      pserver_g->stop();
    fcf::net::uninitialize();
    exit(0);
  }

  int wmain(int a_argc, const wchar_t* a_argv[]){
    try {
      if (a_argc < 2)
        throw std::runtime_error("Config file not specified for command line parameter");
      if (::wcscmp(a_argv[1], L"--unlock") == 0){
        fcf::net::unlock_global_mutex();
        return 0;
      }
      run(std::filesystem::path(a_argv[1]));
      signal(SIGINT, int_signal_handler);
      while (!pserver_g->is_exit())
        std::this_thread::sleep_for(std::chrono::milliseconds(100));
    }
    catch (const std::exception& e) {
      fcf::err << "[Failed start] " << e.what();
      try {
        if (pserver_g.get())
          pserver_g->stop();
      } catch(...) {
      }
      fcf::net::uninitialize();
      exit(1);
      return 1;
    }
    fcf::net::uninitialize();
    return 0;
  }

#else

  #include <sys/types.h>
  #include <sys/wait.h>


  void int_signal_handler(int) {
    if (pserver_g.get())
      pserver_g->stop();
    fcf::net::uninitialize();
    exit(0);
  }

  void broken_pipe_signal_handler(int){
  }

  int main(int a_argc, const char* a_argv[]) {
    signal(SIGPIPE, broken_pipe_signal_handler);
    try {
      if (a_argc < 2)
        throw std::runtime_error("Config file not specified for command line parameter");
      if (::strcmp(a_argv[1], "--unlock") == 0){
        fcf::net::unlock_global_mutex();
        return 0;
      }
      run(std::filesystem::path(a_argv[1]));
      signal(SIGINT, int_signal_handler);
      while (!pserver_g->is_exit())
        std::this_thread::sleep_for(std::chrono::milliseconds(100));
    } catch (const std::exception& e) {
      fcf::err << "[Failed start] " << e.what();
      try {
        if (pserver_g.get()){
          pserver_g->stop();
        }
      } catch(...) {
      }
      fcf::net::uninitialize();
      exit(1);
      return 1;
    }
    fcf::net::uninitialize();
    return 0;
  }

#endif
