#include <vector>
#include <memory>
#include <thread>
#include <mutex>
#include <fcf/net/socket.hpp>
#include "endpoint.hpp"
#include "configuration.hpp"
#include "event_server.hpp"

namespace application {

  class event_server_t;
  class server_t
  {
    friend class event_server_t;
  public:
    server_t(const server_configuration_t& a_config, const global_configuration_t& a_global_config);
    ~server_t();
    void                          run();
    void                          stop();
    bool                          is_stop();
    bool                          is_exit();
    event_server_t*               get_event_server();
    const server_configuration_t& get_config();

  private:
    void _restart_processes(std::string a_handler, const std::string& a_cause);
    void _get_processes_for_message(std::vector<process_identification_t>& a_dst,
                                    bool a_only_active_processes,
                                    bool a_active_flag,
                                    int a_destenition_pair,
                                    std::string a_handler,
                                    std::string a_ignore_handler,
                                    int a_ignore_process);
    void _release_processes_for_message(const std::vector<process_identification_t>& a_procs, bool a_active);

    std::mutex                      _start_stop_mutex;
    bool                            _stop;
    bool                            _exit;
    server_configuration_t          _config;
    global_configuration_t          _global_config;
    std::vector<endpoint_sptr_t>    _endpoints;
    fcf::net::socket_sptr_t         _socket;
    std::shared_ptr<event_server_t> _event_server;

  };

  typedef std::shared_ptr<server_t> server_sptr_t;

}
